package cn.timer.api.controller.crm.task;

import cn.hutool.core.date.DateUtil;
import cn.timer.api.bean.crm.CrmClientAssociate;
import cn.timer.api.bean.crm.CrmClientData;
import cn.timer.api.bean.crm.CrmClientPlan;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.dao.crm.CrmClientDataMapper;
import cn.timer.api.utils.aliyun.AliyunSMS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @Description TODO
 * @Author wuqingjun
 * @Date 2023/5/12
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@Slf4j
public class CrmClientPlanTask {

    @Autowired
    private AliyunSMS aliyunSMS;

    @Resource
    private CrmClientDataMapper crmClientDataMapper;

    @Scheduled(cron = "0 0/30 * * * ? ")
    @Transactional(rollbackFor = Exception.class)
    public void updateInsureStatusTask(){
        try {
            Date now = DateUtil.date();
            List<CrmClientPlan> crmClientPlanList = CrmClientPlan.builder().build().selectList(new QueryWrapper<CrmClientPlan>().lambda().eq(CrmClientPlan::getIfWarnSms,1).eq(CrmClientPlan::getStatus,0).lt(CrmClientPlan::getWarnTime,now));
            if (!CollectionUtils.isEmpty(crmClientPlanList)) {
                for (CrmClientPlan plan:crmClientPlanList) {
                   String result = aliyunSMS.sendCrmClientPlanSms(plan.getCreateUserName(),plan.getCreateUserPhone());
                   if(!StringUtils.isEmpty(result)){
                       CrmClientPlan p = new CrmClientPlan();
                       p.setId(plan.getId());
                       p.setStatus(1);
                       p.updateById();
                       log.info("客户跟进计划定时任务执行-{}-{}",plan.getCreateUserName(),plan.getCreateUserPhone());
                   }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("客户跟进计划定时任务异常");
        }
    }
    /**
     * 每天4点半执行一次
     * 离职员工，全部客户释放公海
     */
    @Scheduled(cron = "0 30 4 * * ?")
    @Transactional(rollbackFor = Exception.class)
    public void lzygCrmClientData() {
        try {
            // 离职用户全部客户
            List<Integer> ids = crmClientDataMapper.selectLzygCrmClientData();
            if(CollectionUtils.isEmpty(ids)){
                return;
            }
            for (Integer id : ids) {
                // 0-进入客户公海
                CrmClientData.builder().id(id).belongUser(0).belongUserName(null).build().updateById();
                // 删除协作人列表
                CrmClientAssociate.builder().build().delete(
                        new QueryWrapper<CrmClientAssociate>().lambda().eq(CrmClientAssociate::getCid, id));
            }
            System.out.println("离职员工，全部客户释放公海:"+ids.size()+"个");
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("离职员工，全部客户释放公海定时任务异常");
        }
    }

}
