package cn.timer.api.controller.insure;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;

import cn.timer.api.bean.insure.InsureProduct;
import cn.timer.api.dao.insure.InsureProductMapper;
import cn.timer.api.dto.insure.PolicyDto;
import cn.timer.api.dto.insure.ProductDto;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import cn.timer.api.utils.aliyun.OSSUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.beust.jcommander.internal.Lists;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


/**
 * 保险产品
 *
 * @author wgd
 * @email 862422848@qq.com
 * @date 2022-03-30 11:36:30
 */
@Api(tags = "8.0保险列表")
@RestController
@Transactional
@RequestMapping(value = "/insureProduct", produces = {"application/json"})
public class InsureProductController {
    @Autowired
    private InsureProductMapper insureProductMapper;
    @Autowired
    private OSSUtil oss;

    @GetMapping(value = "/productList")
    @ApiOperation(value = "12.产品列表", httpMethod = "GET", notes = "产品列表")
    public Result<Object> productList() {
        List<InsureProduct> productList = InsureProduct.builder().build().selectList(new QueryWrapper<InsureProduct>().lambda().eq(InsureProduct::getIsDel, 0));
        return ResultUtil.data(productList);
    }

    @PostMapping(value = "/saveProduct")
    @ApiOperation(value = "保存产品", httpMethod = "POST", notes = "保存产品")
    public Result<Object> saveProduct(@RequestBody InsureProduct insureProduct) {
        insureProduct.setCreateTime(new Date());
        if (insureProduct.insert()) {
            return ResultUtil.data("添加产品成功");
        }
        return ResultUtil.error("添加产品失败");
    }
    @PostMapping(value = "/importTmp")
    @ApiOperation(value = "导入模板", httpMethod = "POST", notes = "保存产品")
    public Result<Object> saveProduct( @RequestParam("productId") Integer productId, @RequestParam("file") MultipartFile file) {
        InsureProduct insureProduct=InsureProduct.builder().id(productId).build().selectById();
        if(insureProduct==null){
            return ResultUtil.error("导入模板失败");
        }
        String path = "8timer2.0/" + productId+ "/Temp/"+ file.getOriginalFilename();
        if (file == null || file.getSize() <= 0) {
            return ResultUtil.error("上传的文件为空，请重新选择！");
        } else {
            try {
              String url=  oss.uploadFile(new File(path), file.getInputStream());
              insureProduct.setTmpUrl(url);
              insureProduct.updateById();
            } catch (IOException e) {
                e.printStackTrace();
            }
            return ResultUtil.data(path, "上传成功！");
        }
    }

}
