package cn.timer.api.controller.insure;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import cn.timer.api.bean.insure.InsurePolicy;
import cn.timer.api.bean.insure.InsureProductPlan;
import cn.timer.api.bean.insure.InsureUser;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.dao.insure.InsureUserMapper;
import cn.timer.api.dto.insure.InsureDto;
import cn.timer.api.dto.insure.InsureUserDto;
import cn.timer.api.dto.insure.PolicyDto;
import cn.timer.api.utils.ExcelUtils;
import cn.timer.api.utils.Page;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.beust.jcommander.internal.Lists;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * 被保人
 *
 * @author wgd
 * @email 862422848@qq.com
 * @date 2022-03-11 09:06:53
 */
@Api(tags = "8.0保险列表")
@RestController
@Transactional
@RequestMapping(value ="/insureUser", produces = {"application/json"})
public class InsureUserController{
    @Autowired
    private InsureUserMapper insureUserMapper;

    @PostMapping(value = "/policyList")
    @ApiOperation(value = "3.保单列表--运营后台", httpMethod = "POST", notes = "保单列表")
    public Result<Object> policyList(@RequestBody PolicyDto policyDto) {
        Map map = Maps.newHashMap();
        List<PolicyDto> list = insureUserMapper.selectPolicyList(policyDto);
        map.put("list", Optional.ofNullable(list).orElse(Lists.newArrayList()));
        map.put("total", insureUserMapper.totalUser(policyDto));
        return ResultUtil.data(map);
    }
    @GetMapping(value = "/userDetial")
    @ApiOperation(value = "被保人详情", httpMethod = "POST", notes = "保单列表")
    public Result<Object> userDetial(@RequestParam("userId") String userId) {
        InsureUser user = InsureUser.builder().id(Integer.parseInt(userId)).build().selectById();
            InsureProductPlan insureProductPlan = InsureProductPlan.builder().id(user.getProductPlanId()).build().selectById();
            user.setPlanIds(JSON.parseArray(insureProductPlan.getPlanJson(), String.class));
            user.setProductName(insureProductPlan.getName());
            user.setBenefitBasicPlan(String.valueOf(insureProductPlan.getId()));
        return ResultUtil.data(user);
    }

    @PostMapping(value = "/userPolicyList")
    @ApiOperation(value = "3.保单列表--8小时端", httpMethod = "POST", notes = "保单列表")
    public Result<Object> userPolicyList(@CurrentUser UserBean userBean, @RequestBody PolicyDto policyDto) {
        Map map = Maps.newHashMap();
        if(!StringUtils.isNullOrEmpty(policyDto.getCreateTime())){
            List arr= JSONObject.parseArray(policyDto.getCreateTime(), String.class);
            policyDto.setPolicyDateStart(arr.get(0).toString());
            policyDto.setPolicyDateEnd(arr.get(1).toString());
        }
        policyDto.setOrgCode(String.valueOf(userBean.getOrgCode()));
        List<PolicyDto> list = insureUserMapper.selectPolicyList(policyDto);
        map.put("list", Optional.ofNullable(list).orElse(Lists.newArrayList()));
        map.put("total", insureUserMapper.totalUser(policyDto));
        return ResultUtil.data(map);
    }
    @PostMapping(value = "/getPolicyUserList")
    @ApiOperation(value = "获取选中用户方案信息", httpMethod = "POST", notes = "获取选中用户方案信息")
    public Result<Object> getPolicyUserList(@RequestBody InsureDto dto){
        List<InsureUserDto> userList= insureUserMapper.selectPlansListByIds(dto.getOldIds());
        if(userList.size()<=0){
            return ResultUtil.error("用户方案错误");
        }
        return ResultUtil.data(userList);
    }
    @GetMapping(value = "/getPolicyPlansList")
    @ApiOperation(value = "获取方案列表", httpMethod = "GET", notes = "获取选中用户方案信息")
    public Result<Object> getPolicyUserList(@RequestParam("policyId") String policyId){
        List<InsureUserDto> userList= insureUserMapper.selectPlansListById(policyId);
        if(userList.size()<=0){
            return ResultUtil.error("用户方案错误");
        }
        return ResultUtil.data(userList);
    }


    @GetMapping(value = "/exportUserList")
    @ApiOperation(value = "导出列表", httpMethod = "GET", notes = "导出列表")
    public void exportUserList(@RequestParam("policyId") String policyId,
                               @RequestParam("status") String status,
                               @RequestParam("name") String name,
                               @RequestParam("policyDateStart") String policyDateStart,
                               @RequestParam("planId") String planId,
                               @RequestParam("categoryId") String categoryId, HttpServletRequest request, HttpServletResponse resp){
        LocalDate localDate = LocalDate.now();
        PolicyDto policyDto = new PolicyDto();
        policyDto.setId(Integer.parseInt(policyId));
        policyDto.setStatus(status);
        policyDto.setName(name);
        policyDto.setPolicyDateStart(policyDateStart);
        policyDto.setPlanId(planId);
        policyDto.setCategoryId(categoryId);
//        policyDto.setPage(null);
        policyDto.setAll(true);
        List<PolicyDto> userList = insureUserMapper.selectPolicyList(policyDto);
        String[] rowName = new String[]{"编号", "姓名", "证件类型", "证件号", "保单号", "状态", "计划", "保险生效时间", "保险失效时间"};

        XSSFWorkbook workbook = null;

            /*
                1，创建工作簿对象，然后创建大标题行，并设置标题
             */
        // 创建工作簿对象
        workbook = new XSSFWorkbook();
        // 创建一个表格对象
        XSSFSheet sheet = workbook.createSheet("Sheet1");

        String mark = "title";
        // 定义大标题行的样式
        XSSFCellStyle titleCellStyle = ExcelUtils.getCellStyle(workbook, mark);
        String title="导出线下人员名单";
        // 如果参数title不等空，则设置Sheet表格的大标题
        if (!"null".equals(title) && title != null
                && !"".equals(title) && !title.trim().isEmpty()) {
            // 创建表格大标题行
            XSSFRow titleRow = sheet.createRow(0);
            // 创建表格大标题行的第一个单元格
            XSSFCell titleCell = titleRow.createCell(0);

            // 定义大标题行的宽度和高度（合并单元格）
            sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, (rowName.length - 1)));
            // 设置大标题行的单元格样式
            titleCell.setCellStyle(titleCellStyle);
            // 设置大标题行的单元格名称
            titleCell.setCellValue(title);
        }

        int columnNum = rowName.length;
        // 创建小标题行，由于0行和1行用作大标题行，所以小标题行从2开始
        XSSFRow subTitleRow = sheet.createRow(2);

        // 将列头设置到sheet的单元格中
        for (int i = 0; i < columnNum; i++) {
            // 创建小标题行的单元格
            XSSFCell subTitleCell = subTitleRow.createCell(i);
            // 设置单元格的单元格类型
            subTitleCell.setCellType(CellType.STRING);
            // 使用数组中的数据作为单元格的文本来创建小标题
            XSSFRichTextString text = new XSSFRichTextString(rowName[i]);
            // 设置小标题单元格的样式
            subTitleCell.setCellStyle(titleCellStyle);
            // 设置文本到小标题单元格中
            subTitleCell.setCellValue(text);
        }
        XSSFCell cell=null;
        for (int i = 0; i < userList.size(); i++) {
            PolicyDto objArr = userList.get(i);
            // 创建当前要填充数据的行对象
            XSSFRow currentRow = sheet.createRow(i + 3);
            //第i+1行 第一列
            cell = currentRow.createCell(0, CellType.NUMERIC);
            cell.setCellValue(i+1);
            cell = currentRow.createCell(1);
            cell.setCellValue(objArr.getName());
            cell = currentRow.createCell(2);
            cell.setCellValue("身份证");
            cell = currentRow.createCell(3);
            cell.setCellValue(objArr.getZjNum());
            cell = currentRow.createCell(4);
            cell.setCellValue(objArr.getPolicyNo());
            cell = currentRow.createCell(5);
            cell.setCellValue(objArr.getStatus().equals("1")?"生效":"失效");
            cell = currentRow.createCell(6);
            cell.setCellValue(objArr.getPlanName());
            cell = currentRow.createCell(7);
            cell.setCellValue(objArr.getPolicyDateStart());
            cell = currentRow.createCell(8);
            cell.setCellValue(objArr.getPolicyDateEnd());
        }
        OutputStream os = null;
        try {
            resp.setContentType("application/octet-stream");
            resp.setHeader("content-disposition", "attachment; filename=" + URLEncoder.encode(localDate+"_"+userList.size()+"名人员清单.xlsx", "UTF-8"));
            resp.setCharacterEncoding("UTF-8");
            os = resp.getOutputStream();
            workbook.write(os);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                os.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
